//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

// This file isn't generated, but this comment is necessary to exclude it from StyleCop analysis.
// <auto-generated/>

using System;
using System.Security.Cryptography;

namespace Microsoft.Data.Encryption.Cryptography
{
    /// <summary>
    /// An encryption key that is used to encrypt and decrypt data.
    /// </summary>
    internal class PlaintextDataEncryptionKey : DataEncryptionKey
    {
        /// <summary>
        /// A local cache to hold previously created <see cref="PlaintextDataEncryptionKey"/> objects for reuse.
        /// </summary>
        /// <remarks>
        /// The retension period is defined by the <see cref="TimeToLive"/> property.
        /// </remarks>
        private static readonly LocalCache<Tuple<string, string>, PlaintextDataEncryptionKey> encryptionKeyCache
            = new LocalCache<Tuple<string, string>, PlaintextDataEncryptionKey>() { TimeToLive = TimeSpan.FromHours(2) };

        /// <summary>
        /// Sets an absolute expiration time, relative to now.
        /// </summary>
        /// <remarks>
        /// The default is 2 hours. Setting the <see cref="TimeToLive"/> will apply to every new
        /// object instantiated by the <see cref="GetOrCreate(string, byte[])"/> method after this
        /// <see cref="TimeToLive"/> change and before the next <see cref="TimeToLive"/> change.
        /// If the object returned by the <see cref="GetOrCreate(string, byte[])"/> method already
        /// existed in the cache before this <see cref="TimeToLive"/> change (cache hit) then the returned
        /// object will have the <see cref="TimeToLive"/> value attributed to it that existed at the time
        /// it was originally created.
        /// </remarks>
        public static TimeSpan TimeToLive
        {
            get => encryptionKeyCache.TimeToLive.Value;
            set => encryptionKeyCache.TimeToLive = value;
        }

        /// <summary>
        /// Returns a cached instance of the <see cref="PlaintextDataEncryptionKey"/> or, if not present, creates a new one
        /// </summary>
        /// <param name="name">The name by which the <see cref="PlaintextDataEncryptionKey"/> will be known.</param>
        /// <param name="unencryptedKey">The unencrypted <see cref="PlaintextDataEncryptionKey"/> value.</param>
        /// <returns>An <see cref="PlaintextDataEncryptionKey"/> object.</returns>
        public static PlaintextDataEncryptionKey GetOrCreate(string name, byte[] unencryptedKey)
        {
            name.ValidateNotNullOrWhitespace(nameof(name));
            unencryptedKey.ValidateNotNullOrEmpty(nameof(unencryptedKey));

            return encryptionKeyCache.GetOrCreate(
                key: Tuple.Create(name, unencryptedKey.ToHexString()),
                createItem: () => new PlaintextDataEncryptionKey(name, unencryptedKey)
            );
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ProtectedDataEncryptionKey"/> class.
        /// </summary>
        /// <param name="name">The name by which the <see cref="PlaintextDataEncryptionKey"/> will be known.</param>
        /// <remarks>
        /// Generates a new 256 bit cryptographically strong random key.
        /// </remarks>
        public PlaintextDataEncryptionKey(string name) : base(name, GenerateNewColumnEncryptionKey()) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ProtectedDataEncryptionKey"/> class.
        /// </summary>
        /// <param name="name">The name by which the <see cref="PlaintextDataEncryptionKey"/> will be known.</param>
        /// <param name="unencryptedKey">The unencrypted <see cref="PlaintextDataEncryptionKey"/> value.</param>
        public PlaintextDataEncryptionKey(string name, byte[] unencryptedKey) : base(name, unencryptedKey) { }

        private static byte[] GenerateNewColumnEncryptionKey()
        {
            byte[] plainTextColumnEncryptionKey = new byte[32];
            RandomNumberGenerator rng = RandomNumberGenerator.Create();
            rng.GetBytes(plainTextColumnEncryptionKey);

            return plainTextColumnEncryptionKey;
        }

        /// <summary>
        /// Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="obj">An object to compare with this object.</param>
        /// <returns>true if the current object is equal to the other parameter; otherwise, false.</returns>
        public override bool Equals(object obj)
        {
            if (!(obj is PlaintextDataEncryptionKey other))
            {
                return false;
            }

            return Name.Equals(other.Name) && RootKeyEquals(other);
        }

        /// <summary>
        /// Generates a hash code for the current object.
        /// </summary>
        /// <returns>A hash code for the current object.</returns>
        public override int GetHashCode() => Tuple.Create(Name, rootKeyHexString).GetHashCode();
    }
}
