//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

// This file isn't generated, but this comment is necessary to exclude it from StyleCop analysis.
// <auto-generated/>

using System;
using System.Data.SqlTypes;
using System.Linq;

using static System.BitConverter;
using static Microsoft.Data.Encryption.Resources.Strings;

namespace Microsoft.Data.Encryption.Cryptography.Serializers
{
    /// <summary>
    /// Contains the methods for serializing and deserializing <see cref="DateTime"/> type data objects
	/// that is compatible with the Always Encrypted feature in SQL Server and Azure SQL.
    /// </summary>
    internal sealed class SqlDateTimeSerializer : Serializer<DateTime>
    {
        private static readonly DateTime MinValue = DateTime.Parse("1753-01-01 00:00:00");

        /// <summary>
        /// The <see cref="Identifier"/> uniquely identifies a particular Serializer implementation.
        /// </summary>
        public override string Identifier => "SQL_DateTime";

        /// <summary>
        /// Deserializes the provided <paramref name="bytes"/>
        /// </summary>
        /// <param name="bytes">The data to be deserialized</param>
        /// <returns>The serialized data</returns>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// <paramref name="bytes"/> is null.
        /// -or-
        /// The length of <paramref name="bytes"/> is less than 8.
        /// </exception>
        public override DateTime Deserialize(byte[] bytes)
        {
            const int SizeOfDate = 4;
            const int SizeOfTime = 4;
            const int SizeOfDateTime = SizeOfDate + SizeOfTime;

            bytes.ValidateNotNull(nameof(bytes));
            bytes.ValidateSize(SizeOfDateTime, nameof(bytes));

            int dayTicks = ToInt32(bytes.Take(SizeOfDate).ToArray(), 0);
            int timeTicks = ToInt32(bytes.Skip(SizeOfDate).Take(SizeOfTime).ToArray(), 0);
            SqlDateTime sqlDateTime = new SqlDateTime(dayTicks, timeTicks);
            return sqlDateTime.Value;
        }

        /// <summary>
        /// Serializes the provided <paramref name="value"/>
        /// </summary>
        /// <param name="value">The value to be serialized</param>
        /// <returns>
        /// An array of bytes with length 8.
        /// </returns>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// <paramref name="value"/> is out of range.
        /// </exception>
        public override byte[] Serialize(DateTime value)
        {
            if (value < MinValue)
            {
                throw new MicrosoftDataEncryptionException(ValueOutOfRange.Format(value));
            }

            SqlDateTime sqlDateTime = new SqlDateTime(value);
            byte[] dayTicks = GetBytes(sqlDateTime.DayTicks);
            byte[] timeTicks = GetBytes(sqlDateTime.TimeTicks);
            return dayTicks.Concat(timeTicks).ToArray();
        }
    }
}
