//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

// This file isn't generated, but this comment is necessary to exclude it from StyleCop analysis.
// <auto-generated/>

using System;
using System.Data.SqlTypes;
using System.Linq;

using static System.BitConverter;
using static Microsoft.Data.Encryption.Resources.Strings;

namespace Microsoft.Data.Encryption.Cryptography.Serializers
{
    /// <summary>
    /// Contains the methods for serializing and deserializing <see cref="DateTime"/> type data objects
    /// that is compatible with the Always Encrypted feature in SQL Server and Azure SQL Database.
    /// </summary>

    internal sealed class SqlSmallDateTimeSerializer : Serializer<DateTime>
    {
        private static readonly DateTime MinValue = DateTime.Parse("1900-01-01 00:00:00");
        private static readonly DateTime MaxValue = DateTime.Parse("2079-06-06 23:59:59");

        /// <summary>
        /// The <see cref="Identifier"/> uniquely identifies a particular Serializer implementation.
        /// </summary>
        public override string Identifier => "SQL_SmallDateTime";

        /// <summary>
        /// Deserializes the provided <paramref name="bytes"/>
        /// </summary>
        /// <param name="bytes">The data to be deserialized</param>
        /// <returns>The serialized data</returns>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// <paramref name="bytes"/> is null.
        /// -or-
        /// The length of <paramref name="bytes"/> is less than 4.
        /// </exception>
        public override DateTime Deserialize(byte[] bytes)
        {
            const int SizeOfDate = 2;
            const int SizeOfTime = 2;
            const int SizeOfDateTime = SizeOfDate + SizeOfTime;

            bytes.ValidateNotNull(nameof(bytes));
            bytes.ValidateSize(SizeOfDateTime, nameof(bytes));

            int dayTicks = ToUInt16(bytes.Take(SizeOfDate).ToArray(), 0);
            int timeTicks = ToUInt16(bytes.Skip(SizeOfDate).Take(SizeOfTime).ToArray(), 0) * SqlDateTime.SQLTicksPerMinute;
            SqlDateTime sqlDateTime = new SqlDateTime(dayTicks, timeTicks);
            return sqlDateTime.Value;
        }

        /// <summary>
        /// Serializes the provided <paramref name="value"/>
        /// </summary>
        /// <param name="value">The value to be serialized</param>
        /// <returns>
        /// An array of bytes with length 4.
        /// </returns>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// <paramref name="value"/> is out of range.
        /// </exception>
        public override byte[] Serialize(DateTime value)
        {
            if (value < MinValue || value > MaxValue)
            {
                throw new MicrosoftDataEncryptionException(ValueOutOfRange.Format(value));
            }

            SqlDateTime sqlDateTime = new SqlDateTime(value.AddSeconds(30));
            byte[] dayTicks = GetBytes((short)sqlDateTime.DayTicks);
            byte[] timeTicks = GetBytes((short)(sqlDateTime.TimeTicks / SqlDateTime.SQLTicksPerMinute));
            return dayTicks.Concat(timeTicks).ToArray();
        }
    }
}
